@@ -23,6 +23,7 @@
 #include <stdbool.h>
 #include <stddef.h>
 #include <unistd.h>
+#include "src/defs.h"
 
 /*
  * Information elements, IEEE Std 802.11-2012 ch. 8.4.2 and
@@ -361,6 +362,11 @@ enum ie_rsn_akm_suite {
 	IE_RSN_AKM_SUITE_OSEN			= 0x40000,
 };
 
+static inline bool IE_AKM_IS_OWE(uint32_t akm)
+{
+	return akm & (IE_RSN_AKM_SUITE_OWE);
+}
+
 static inline bool IE_AKM_IS_SAE(uint32_t akm)
 {
 	return akm & (IE_RSN_AKM_SUITE_SAE_SHA256 |
@@ -560,7 +566,7 @@ struct ie_fils_ip_addr_response_info {
 
 struct ie_owe_transition_info {
 	uint8_t bssid[6];
-	uint8_t ssid[32];
+	uint8_t ssid[SSID_MAX_SIZE];
 	size_t ssid_len;
 	uint8_t oper_class;
 	uint8_t channel;
@@ -592,6 +598,23 @@ static inline const unsigned char *ie_tlv_iter_get_data(
 	return iter->data;
 }
 
+static inline bool ie_tlv_iter_data_eq(struct ie_tlv_iter *a,
+					struct ie_tlv_iter *b)
+{
+	if (a == b)
+		return true;
+
+	if (a == NULL || b == NULL)
+		return false;
+
+	if (ie_tlv_iter_get_length(a) != ie_tlv_iter_get_length(b))
+		return false;
+
+	return memcmp(ie_tlv_iter_get_data(a),
+			ie_tlv_iter_get_data(b),
+			ie_tlv_iter_get_length(a)) == 0;
+}
+
 void *ie_tlv_extract_wsc_payload(const uint8_t *ies, size_t len,
 							ssize_t *out_len);
 void *ie_tlv_encapsulate_wsc_payload(const uint8_t *data, size_t len,
